@@ -194,7 +194,7 @@ LocalSubMatch::get_postlist(MultiMatch * matcher,
 {
     LOGCALL(MATCH, PostList*, "LocalSubMatch::get_postlist", matcher | total_subqs_ptr | Literal("[total_subqs]"));
 
-    if (query.empty())
+    if (query.empty() || db->get_doccount() == 0)
 	RETURN(new EmptyPostList); // MatchNothing
 
     // Build the postlist tree for the query.  This calls
@@ -271,7 +271,7 @@ LocalSubMatch::open_post_list(const string& term,
 	    !wt_factory->get_sumpart_needs_wdf_()) {
 	    Xapian::doccount sub_tf;
 	    db->get_freqs(term, &sub_tf, NULL);
-	    if (sub_tf == db->get_doccount()) {
+	    if (sub_tf == qopt->db_size) {
 		// If we're not going to use the wdf or term positions, and the
 		// term indexes all documents, we can replace it with the
 		// MatchAll postlist, which is especially efficient if there
@@ -300,7 +300,7 @@ LocalSubMatch::open_post_list(const string& term,
 	auto res = stats->termfreqs.emplace(term, TermFreqs());
 	if (res.second) {
 	    // Term came from a wildcard, but the same term may be elsewhere
-	    // in the query so only accumulates its TermFreqs if emplace()
+	    // in the query so only accumulate its TermFreqs if emplace()
 	    // created a new element.
 	    db->get_freqs(term,
 			  &res.first->second.termfreq,
